searchData={"items":[{"type":"module","title":"Nerves.Runtime","doc":"Nerves.Runtime contains functions useful for almost all Nerves-based devices.","ref":"Nerves.Runtime.html"},{"type":"function","title":"Nerves.Runtime.cmd/3","doc":"Run system command and log output into logger.\n\nNOTE: Unlike System.cmd/3, this does not raise if the executable isn't found","ref":"Nerves.Runtime.html#cmd/3"},{"type":"function","title":"Nerves.Runtime.firmware_valid?/0","doc":"Return whether the firmware has been marked as valid\n\nSince \"valid\" means that the next boot will run the same firmware, this also\nreturns `true` if firmware validation isn't in use.\n\nSee `validate_firmware/0` for more information.","ref":"Nerves.Runtime.html#firmware_valid?/0"},{"type":"function","title":"Nerves.Runtime.halt/0","doc":"Halt the device (meaning hang, not power off, nor reboot).\n\nNote: this is different than :erlang.halt(), which exits BEAM, and may end up\nrebooting the device if `erlinit.config` settings allow reboot on exit.","ref":"Nerves.Runtime.html#halt/0"},{"type":"function","title":"Nerves.Runtime.mix_target/0","doc":"Return the mix target that was used to build this firmware\n\nIf you're running on the development machine, this will return `:host`.\nIf not, it will return whatever the user specified with the `MIX_TARGET`\nenvironment variable when building this firmware.","ref":"Nerves.Runtime.html#mix_target/0"},{"type":"function","title":"Nerves.Runtime.poweroff/0","doc":"Power off the device.\n\nThis calls `:init.stop/0` internally. If `:init.stop/0` takes longer than the\n`erlinit.config`'s `--graceful-powerdown` setting (likely 10 seconds) then\nthe system will be hard rebooted.","ref":"Nerves.Runtime.html#poweroff/0"},{"type":"function","title":"Nerves.Runtime.reboot/0","doc":"Reboot the device and gracefully shutdown the Erlang VM.\n\nThis calls `:init.stop/0` internally. If `:init.stop/0` takes longer than the\n`erlinit.config`'s `--graceful-powerdown` setting (likely 10 seconds) then\nthe system will be hard rebooted.","ref":"Nerves.Runtime.html#reboot/0"},{"type":"function","title":"Nerves.Runtime.revert/1","doc":"Revert the device to running the previous firmware\n\nThis switches the active firmware slot back to the previous one and then\nreboots. This fails if the slot is empty or partially overwritten to prevent\naccidents. It also requires the revert feature to be implemented in the\nNerves system that's in use. See `Nerves.Runtime.FwupOps` for how this works.\n\nSpecifying `reboot: false` is allowed, but be sure to reboot. It's easy to\nget confused if you don't reboot afterwards and do a double revert or\nsomething else silly.","ref":"Nerves.Runtime.html#revert/1"},{"type":"function","title":"Nerves.Runtime.serial_number/0","doc":"Return the device's serial number\n\nSerial number storage is device-specific and configurable. Serial numbers can\nbe programmed in one-time programmable locations like in CPU ROM or\ncryptographic elements. They can also be in rewritable locations like a\nU-Boot environment block.\n\nNerves uses the [`boardid`](https://github.com/nerves-project/boardid/) by\ndefault (set `:boardid_path` key in the application environment to another\nprogram to override). Boardid uses the `/etc/boardid.config` file to\ndetermine how to read the serial number. Official Nerves systems provide\nreasonable default mechanisms for getting started. Override this file in your\napplication's `rootfs_overlay` to customize it.\n\nThis function never raises. If a serial number isn't available for any\nreason, it will return a serial number of `\"unconfigured\"`.","ref":"Nerves.Runtime.html#serial_number/0"},{"type":"function","title":"Nerves.Runtime.validate_firmware/0","doc":"Mark the running firmware as valid\n\nA device cannot receive a new firmware if the current one has not been validated.\nIn the official Nerves systems, this typically happens automatically. If you are\nhandling the firmware validation in your app, then this function can be used as\na helper to mark firmware as valid.\n\nFor systems that support automatic reverting, if the firmware is not marked as\nvalid, then the next reboot will cause a revert to the old firmware","ref":"Nerves.Runtime.html#validate_firmware/0"},{"type":"module","title":"Nerves.Runtime.FwupOps","doc":"Convenience functions for /usr/share/fwup/ops.fw\n\nThe `/usr/share/fwup/ops.fw` is provided by the Nerves system for handling\nsome eMMC/MicroSD card operations. Look for `fwup-ops.conf` in the Nerves\nsystem source tree for more details. It used to be called\n`revert.fw`/`fwup-revert.conf` when it only handled reverting which firmware\nimage was active.","ref":"Nerves.Runtime.FwupOps.html"},{"type":"function","title":"Nerves.Runtime.FwupOps.factory_reset/1","doc":"Reset the application data partition to its original state\n\nThis clears out the application data partition at a low level so that it will\nbe reformatted on the next boot. If all application settings are stored on\nthe partition, then this will be like a factory reset. Be aware that many\nsettings are stored on the application data partition including network\nsettings like WiFi SSIDs and passwords. Factory reset devices may not connect\nto the network afterwards.","ref":"Nerves.Runtime.FwupOps.html#factory_reset/1"},{"type":"function","title":"Nerves.Runtime.FwupOps.prevent_revert/0","doc":"Make it impossible to revert to the other partition\n\nThis wipes the opposite firmware partition and clears out metadata for it.\nAttempts to revert will fail. This is useful if loading a special firmware\ntemporarily that shouldn't be used again even accidentally.","ref":"Nerves.Runtime.FwupOps.html#prevent_revert/0"},{"type":"function","title":"Nerves.Runtime.FwupOps.revert/1","doc":"Revert to the previous firmware\n\nThis invokes the \"revert\" task in the `ops.fw` and then reboots (unless told\notherwise).  The revert task switches the active firmware partition to the\nopposite one so that future reboots use the previous firmware.","ref":"Nerves.Runtime.FwupOps.html#revert/1"},{"type":"function","title":"Nerves.Runtime.FwupOps.validate/0","doc":"Validate the current partition\n\nFor Nerves systems that support automatic rollback of firmware versions, this\nmarks the partition as good so that it will continue to be used on future\nboots.\n\nCall `Nerves.Runtime.validate_firmware/0` instead.","ref":"Nerves.Runtime.FwupOps.html#validate/0"},{"type":"type","title":"Nerves.Runtime.FwupOps.options/0","doc":"General options for utilities\n\n* `:reboot` - Call `Nerves.Runtime.reboot/0` after running (defaults to\n `true` on destructive operations)","ref":"Nerves.Runtime.FwupOps.html#t:options/0"},{"type":"module","title":"Nerves.Runtime.Heart","doc":"Functions for querying Nerves Heart and the device's watchdog\n\nNerves Heart integrates Erlang's\n[heart](https://www.erlang.org/doc/man/heart.html) process with a hardware\nwatchdog.  This makes it possible for a device to recover from a hang. The\nway it works is that the Erlang runtime regularly checks that it's ok. If so,\nit sends a message to `heart`. Nerves heart then pets the hardware watchdog.\nIf messages ever stop being sent to `heart`, the hardware watchdog will trip\nand reboot the device. You can add additional health checks for your\napplication by providing a callback to `:heart.set_callback/2`.\n\nSee [nerves_heart](https://github.com/nerves-project/nerves_heart) for more\ninformation.","ref":"Nerves.Runtime.Heart.html"},{"type":"function","title":"Nerves.Runtime.Heart.guarded_poweroff/0","doc":"Initiate a poweroff that's guarded by the hardware watchdog\n\nMost users should call `Nerves.Runtime.poweroff/0` instead which calls this\nand shuts down the Erlang VM.\n\nSupport with Nerves Heart v2.0 and later.","ref":"Nerves.Runtime.Heart.html#guarded_poweroff/0"},{"type":"function","title":"Nerves.Runtime.Heart.guarded_reboot/0","doc":"Initiate a reboot that's guarded by the hardware watchdog\n\nMost users should call `Nerves.Runtime.reboot/0` instead which calls this and\nshuts down the Erlang VM.\n\nSupport with Nerves Heart v2.0 and later.","ref":"Nerves.Runtime.Heart.html#guarded_reboot/0"},{"type":"function","title":"Nerves.Runtime.Heart.init_complete/0","doc":"Notify Nerves heart that initialization is complete\n\nThis can be used to ensure that the code that calls `:heart.set_callback/2`\ngets run. To use, add the following to your projects `rel/vm.args.eex`:\n\n```text","ref":"Nerves.Runtime.Heart.html#init_complete/0"},{"type":"function","title":"Require an initialization handshake within 15 minutes - Nerves.Runtime.Heart.init_complete/0","doc":"-env HEART_INIT_TIMEOUT 900\n```\n\nThen call `Nerves.Runtime.Heart.init_complete/0` after\n`:heart.set_callback/2` is called.\n\nSupported by Nerves Heart v2.0 and later","ref":"Nerves.Runtime.Heart.html#init_complete/0-require-an-initialization-handshake-within-15-minutes"},{"type":"function","title":"Nerves.Runtime.Heart.running?/0","doc":"Return whether Nerves heart is running\n\nIf you're using a Nerves device, this always returns `true` except possibly\nwhen porting Nerves to new hardware. It is a quick sanity check.","ref":"Nerves.Runtime.Heart.html#running?/0"},{"type":"function","title":"Nerves.Runtime.Heart.snooze/0","doc":"Snooze heart related reboots for the next 15 minutes\n\nRun this to buy some time if reboots from heart or hardware watchdog are\ngetting in the way.\n\nSupport with Nerves Heart v2.2 and later.","ref":"Nerves.Runtime.Heart.html#snooze/0"},{"type":"function","title":"Nerves.Runtime.Heart.status/0","doc":"Return the current Nerves Heart status\n\nErrors are returned when not running Nerves Heart","ref":"Nerves.Runtime.Heart.html#status/0"},{"type":"function","title":"Nerves.Runtime.Heart.status!/0","doc":"Raising version of status/0","ref":"Nerves.Runtime.Heart.html#status!/0"},{"type":"type","title":"Nerves.Runtime.Heart.info/0","doc":"Nerves Heart's current status\n\nSee [nerves_heart](https://github.com/nerves-project/nerves_heart) for more\ninformation.","ref":"Nerves.Runtime.Heart.html#t:info/0"},{"type":"type","title":"Nerves.Runtime.Heart.info_v1/0","doc":"Nerves Heart v1.x information","ref":"Nerves.Runtime.Heart.html#t:info_v1/0"},{"type":"type","title":"Nerves.Runtime.Heart.info_v2/0","doc":"Nerves Heart v2.x information","ref":"Nerves.Runtime.Heart.html#t:info_v2/0"},{"type":"module","title":"Nerves.Runtime.Init","doc":"GenServer that handles device initialization.\n\nInitialization currently consists of:\n\n1. Mounting the application partition\n2. If the application partition can't be mounted, format it, and then mount it.\n\nDevice initialization is usually a first boot only operation. It's possible\nthat device filesystems get corrupt enough to cause them to be reinitialized.\nSince corruption should be rare, Nerves systems create firmware images\nwithout formatting the application partition. This has the benefit of\nexercising the corruption repair code. It's also required since some\nfilesystem types can only be formatted on device.\n\nLong format times can be problematic in manufacturing. If this is an issue,\nsee if you can use F2FS since it formats much faster than ext4. Some devices\nhave also had stalls when formatting while waiting for enough entropy to\ngenerate a UUID. Look into hardcoding UUIDs or enabling a hw random number\ngenerator to increase entropy.","ref":"Nerves.Runtime.Init.html"},{"type":"function","title":"Nerves.Runtime.Init.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Nerves.Runtime.Init.html#child_spec/1"},{"type":"function","title":"Nerves.Runtime.Init.init_application_partition/0","doc":"","ref":"Nerves.Runtime.Init.html#init_application_partition/0"},{"type":"function","title":"Nerves.Runtime.Init.start_link/1","doc":"","ref":"Nerves.Runtime.Init.html#start_link/1"},{"type":"module","title":"Nerves.Runtime.KV","doc":"Key-Value storage for firmware variables\n\nKV provides functionality to read and modify firmware metadata.\nThe firmware metadata contains information such as the active firmware\nslot, where the application data partition is located, etc. It may also contain\nboard provisioning information like factory calibration so long as it is not\ntoo large.\n\nThe metadata store is a simple key-value store where both keys and values are\nASCII strings. Writes are expected to be infrequent and primarily done\non firmware updates. If you expect frequent writes, it is highly recommended\nto persist the data elsewhere.\n\nThe default KV backend loads and stores metadata to a U-Boot-formatted environment\nblock. This doesn't mean that the device needs to run U-Boot. It just\nhappens to be a convenient data format that's well supported.\n\nThere are some expectations on keys. See the Nerves.Runtime README.md for\nnaming conventions and expected key names. These are not enforced\n\nTo change the KV backend, implement the `Nerves.Runtime.KVBackend` behaviour and\nconfigure the application environment in your\nprogram's `config.exs` like the following:\n\n```elixir\nconfig :nerves_runtime, kv_backend: {MyKeyValueBackend, options}\n```","ref":"Nerves.Runtime.KV.html"},{"type":"module","title":"Examples - Nerves.Runtime.KV","doc":"Getting all firmware metadata:\n\n    iex> Nerves.Runtime.KV.get_all()\n    %{\n      \"a.nerves_fw_application_part0_devpath\" => \"/dev/mmcblk0p3\",\n      \"a.nerves_fw_application_part0_fstype\" => \"ext4\",\n      \"a.nerves_fw_application_part0_target\" => \"/root\",\n      \"a.nerves_fw_architecture\" => \"arm\",\n      \"a.nerves_fw_author\" => \"The Nerves Team\",\n      \"a.nerves_fw_description\" => \"\",\n      \"a.nerves_fw_misc\" => \"\",\n      \"a.nerves_fw_platform\" => \"rpi0\",\n      \"a.nerves_fw_product\" => \"test_app\",\n      \"a.nerves_fw_uuid\" => \"d9492bdb-94de-5288-425e-2de6928ef99c\",\n      \"a.nerves_fw_vcs_identifier\" => \"\",\n      \"a.nerves_fw_version\" => \"0.1.0\",\n      \"b.nerves_fw_application_part0_devpath\" => \"/dev/mmcblk0p3\",\n      \"b.nerves_fw_application_part0_fstype\" => \"ext4\",\n      \"b.nerves_fw_application_part0_target\" => \"/root\",\n      \"b.nerves_fw_architecture\" => \"arm\",\n      \"b.nerves_fw_author\" => \"The Nerves Team\",\n      \"b.nerves_fw_description\" => \"\",\n      \"b.nerves_fw_misc\" => \"\",\n      \"b.nerves_fw_platform\" => \"rpi0\",\n      \"b.nerves_fw_product\" => \"test_app\",\n      \"b.nerves_fw_uuid\" => \"4e08ad59-fa3c-5498-4a58-179b43cc1a25\",\n      \"b.nerves_fw_vcs_identifier\" => \"\",\n      \"b.nerves_fw_version\" => \"0.1.1\",\n      \"nerves_fw_active\" => \"b\",\n      \"nerves_fw_devpath\" => \"/dev/mmcblk0\",\n      \"nerves_serial_number\" => \"\"\n    }\n\nParts of the firmware metadata are global, while others pertain to a\nspecific firmware slot. This is indicated by the key - data which describes\nfirmware of a specific slot have keys prefixed with the name of the\nfirmware slot. In the above example, `\"nerves_fw_active\"` and\n`\"nerves_serial_number\"` are global, while `\"a.nerves_fw_version\"` and\n`\"b.nerves_fw_version\"` apply to the \"a\" and \"b\" firmware slots,\nrespectively.\n\nIt is also possible to get firmware metadata that only pertains to the\ncurrently active firmware slot:\n\n    iex> Nerves.Runtime.KV.get_all_active()\n    %{\n      \"nerves_fw_application_part0_devpath\" => \"/dev/mmcblk0p3\",\n      \"nerves_fw_application_part0_fstype\" => \"ext4\",\n      \"nerves_fw_application_part0_target\" => \"/root\",\n      \"nerves_fw_architecture\" => \"arm\",\n      \"nerves_fw_author\" => \"The Nerves Team\",\n      \"nerves_fw_description\" => \"\",\n      \"nerves_fw_misc\" => \"\",\n      \"nerves_fw_platform\" => \"rpi0\",\n      \"nerves_fw_product\" => \"test_app\",\n      \"nerves_fw_uuid\" => \"4e08ad59-fa3c-5498-4a58-179b43cc1a25\",\n      \"nerves_fw_vcs_identifier\" => \"\",\n      \"nerves_fw_version\" => \"0.1.1\"\n    }\n\nNote that `get_all_active/0` strips out the `a.` and `b.` prefixes.\n\nFurther, the two functions `get/1` and `get_active/1` allow you to get a\nspecific key from the firmware metadata. `get/1` requires specifying the\nentire key name, while `get_active/1` will prepend the slot prefix for you:\n\n    iex> Nerves.Runtime.KV.get(\"nerves_fw_active\")\n    \"b\"\n    iex> Nerves.Runtime.KV.get(\"b.nerves_fw_uuid\")\n    \"4e08ad59-fa3c-5498-4a58-179b43cc1a25\"\n    iex> Nerves.Runtime.KV.get_active(\"nerves_fw_uuid\")\n    \"4e08ad59-fa3c-5498-4a58-179b43cc1a25\"\n\nAside from reading values from the KV store, it is also possible to write\nnew values to the firmware metadata. New values may either have unique keys,\nin which case they will be added to the firmware metadata, or re-use a key,\nin which case they will overwrite the current value with that key:\n\n    iex> :ok = Nerves.Runtime.KV.put(\"my_firmware_key\", \"my_value\")\n    iex> :ok = Nerves.Runtime.KV.put(\"nerves_serial_number\", \"my_new_serial_number\")\n    iex> Nerves.Runtime.KV.get(\"my_firmware_key\")\n    \"my_value\"\n    iex> Nerves.Runtime.KV.get(\"nerves_serial_number\")\n    \"my_new_serial_number\"\n\nIt is possible to write a collection of values at once, in order to\nminimize number of writes:\n\n    iex> :ok = Nerves.Runtime.KV.put(%{\"one_key\" => \"one_val\", \"two_key\" => \"two_val\"})\n    iex> Nerves.Runtime.KV.get(\"one_key\")\n    \"one_val\"\n\nLastly, `put_active/1` and `put_active/2` allow you to write firmware metadata to the\ncurrently active firmware slot without specifying the slot prefix yourself:\n\n    iex> :ok = Nerves.Runtime.KV.put_active(\"nerves_fw_misc\", \"Nerves is awesome\")\n    iex> Nerves.Runtime.KV.get_active(\"nerves_fw_misc\")\n    \"Nerves is awesome\"","ref":"Nerves.Runtime.KV.html#module-examples"},{"type":"function","title":"Nerves.Runtime.KV.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Nerves.Runtime.KV.html#child_spec/1"},{"type":"function","title":"Nerves.Runtime.KV.get/1","doc":"Get the key regardless of firmware slot","ref":"Nerves.Runtime.KV.html#get/1"},{"type":"function","title":"Nerves.Runtime.KV.get_active/1","doc":"Get the key for only the active firmware slot","ref":"Nerves.Runtime.KV.html#get_active/1"},{"type":"function","title":"Nerves.Runtime.KV.get_all/0","doc":"Get all keys regardless of firmware slot","ref":"Nerves.Runtime.KV.html#get_all/0"},{"type":"function","title":"Nerves.Runtime.KV.get_all_active/0","doc":"Get all key value pairs for only the active firmware slot","ref":"Nerves.Runtime.KV.html#get_all_active/0"},{"type":"function","title":"Nerves.Runtime.KV.put/1","doc":"Write a collection of key-value pairs to the firmware metadata","ref":"Nerves.Runtime.KV.html#put/1"},{"type":"function","title":"Nerves.Runtime.KV.put/2","doc":"Write a key-value pair to the firmware metadata","ref":"Nerves.Runtime.KV.html#put/2"},{"type":"function","title":"Nerves.Runtime.KV.put_active/1","doc":"Write a collection of key-value pairs to the active firmware slot","ref":"Nerves.Runtime.KV.html#put_active/1"},{"type":"function","title":"Nerves.Runtime.KV.put_active/2","doc":"Write a key-value pair to the active firmware slot","ref":"Nerves.Runtime.KV.html#put_active/2"},{"type":"function","title":"Nerves.Runtime.KV.start_link/1","doc":"Start the KV store server\n\nOptions:\n* `:kv_backend` - a KV backend of the form `{module, options}` or just `module`","ref":"Nerves.Runtime.KV.html#start_link/1"},{"type":"type","title":"Nerves.Runtime.KV.string_map/0","doc":"The KV store is a string -> string map\n\nSince the KV store is backed by things like the U-Boot environment blocks,\nthe keys and values can't be just any string. For example, characters with\nthe value `0` (i.e., `NULL`) are disallowed. The `=` sign is also disallowed\nin keys. Values may have embedded new lines. In general, it's recommended to\nstick with ASCII values to avoid causing trouble when working with C programs\nwhich also access the variables.","ref":"Nerves.Runtime.KV.html#t:string_map/0"},{"type":"behaviour","title":"Nerves.Runtime.KVBackend","doc":"Behaviour for customizing the Nerves Runtime's key-value store","ref":"Nerves.Runtime.KVBackend.html"},{"type":"callback","title":"Nerves.Runtime.KVBackend.load/1","doc":"Load the KV store and return its contents\n\nThis will be called on boot and should return all persisted key/value pairs.\nThe results will be cached and if a change should be persisted, `c:save/2` will\nbe called with the update.","ref":"Nerves.Runtime.KVBackend.html#c:load/1"},{"type":"callback","title":"Nerves.Runtime.KVBackend.save/2","doc":"Persist the updated KV pairs\n\nThe KV map contains the KV pairs returned by `c:load/1` with any changes made\nby users of `Nerves.Runtime.KV`.","ref":"Nerves.Runtime.KVBackend.html#c:save/2"},{"type":"module","title":"Nerves.Runtime.KVBackend.InMemory","doc":"In-memory KV store\n\nThis KV store keeps everything in memory. Use it by specifying it\nas a backend in the application configuration. Specifying an initial\nset of contents is optional.\n\n```elixir\nconfig :nerves_runtime, :kv_backend, {Nerves.Runtime.KV.InMemory, contents: %{\"key\" => \"value\"}}\n```","ref":"Nerves.Runtime.KVBackend.InMemory.html"},{"type":"module","title":"Nerves.Runtime.KVBackend.UBootEnv","doc":"U-Boot environment block KV store\n\nThis is the default KV store. It delegates to the `UBootEnv` library\nfor loading and saving to a U-Boot formatted environment block. There's\nnothing to configure. It will find the block by reading `/etc/fw_env.config`.","ref":"Nerves.Runtime.KVBackend.UBootEnv.html"},{"type":"extras","title":"nerves_runtime","doc":"# nerves_runtime\n\n[![CircleCI](https://circleci.com/gh/nerves-project/nerves_runtime.svg?style=svg)](https://circleci.com/gh/nerves-project/nerves_runtime)\n[![Hex version](https://img.shields.io/hexpm/v/nerves_runtime.svg \"Hex version\")](https://hex.pm/packages/nerves_runtime)\n\n`nerves_runtime` is a core component of Nerves. It contains applications and\nlibraries that are expected to be useful on all Nerves devices.\n\nHere are its features:\n\n* Generic system and filesystem initialization (suitable for use with\n  [`shoehorn`](https://github.com/nerves-project/shoehorn))\n* Introspection of Nerves system, firmware, and deployment metadata\n* Device reboot and shutdown\n* A small Linux kernel `uevent` application for capturing hardware change events\n  and more. See [`nerves_uevent`](https://github.com/nerves-project/nerves_uevent).\n* Device serial numbers\n* Linux log integration with Elixir. See [`nerves_logging`](https://github.com/nerves-project/nerves_logging)\n\nThe following sections describe the features in more detail. For more\ninformation, see the [hex docs](https://hexdocs.pm/nerves_runtime).","ref":"readme.html"},{"type":"extras","title":"System Initialization - nerves_runtime","doc":"`nerves_runtime` provides an OTP application (`nerves_runtime`) that can\ninitialize the system when it is started. For this to be useful,\n`nerves_runtime` must be started before other OTP applications, since most will\nassume that the system is already initialized before they start. To set up\n`nerves_runtime` to work with `shoehorn`, you will need to do the following:\n\n1. Add `shoehorn` to your `mix.exs` dependency list\n2. Add a `:shoehorn` configuration to `config.exs` and `:nerves_runtime` to the\n   beginning of the `init:` list:\n\n```elixir\nconfig :shoehorn,\n  init: [:nerves_runtime, :other_app1, :other_app2]\n```","ref":"readme.html#system-initialization"},{"type":"extras","title":"Filesystem Initialization - nerves_runtime","doc":"Nerves systems generally ship with one or more application filesystem\npartitions. These are used for persisting data that is expected to live between\nfirmware updates. The root filesystem cannot be used since it is mounted as\nread-only by default.\n\n`nerves_runtime` takes an unforgiving approach to managing the application\npartition: if it can't be mounted as read-write, it gets re-formatted. While\nfilesystem corruption should be a rare event, even with unexpected loss of\npower, Nerves devices may not always be accessible for manual recovery. This\ndefault behavior provides a basic recoverability guarantee.\n\nTo verify that this recovery works, Nerves systems usually leave the application\nfilesystems uninitialized so that the format operation happens on the first\nboot. This means that the first boot takes slightly longer than subsequent\nboots.\n\nA common implementation of \"reset to factory defaults\" is to purposely erase\n(corrupt) the application partition and reboot. See\n`Nerves.Runtime.FwupOps.factory_reset/1`.\n\n`nerves_runtime` uses firmware metadata to determine how to mount and initialize\nthe application partition. The following variables are important:\n\n* `[partition].nerves_fw_application_part0_devpath` - the path to the\n  application partition (e.g. `/dev/mmcblk0p3`)\n* `[partition].nerves_fw_application_part0_fstype` - the type of filesystem\n  (e.g. `ext4`)\n* `[partition].nerves_fw_application_part0_target` - where the partition should\n  be mounted (e.g. `/root` or `/mnt/appdata`)","ref":"readme.html#filesystem-initialization"},{"type":"extras","title":"Nerves System and Firmware Metadata - nerves_runtime","doc":"All official Nerves systems maintain a list of key-value pairs for tracking\nvarious information about the system. This information is not intended to be\nwritten frequently. To get this information, you can call one of the following:\n\n* `Nerves.Runtime.KV.get_all_active/0` - return all key-value pairs associated\n  with the active firmware.\n* `Nerves.Runtime.KV.get_all/0` - return all key-value pairs, including those\n  from the inactive firmware, if any.\n* `Nerves.Runtime.KV.get_active/1` - look up the value of a key associated with\n  the active firmware.\n* `Nerves.Runtime.KV.get/1` - look up the value of a key, including those from\n  the inactive firmware, if any.\n\nGlobal Nerves metadata includes the following:\n\nKey                 | Build Environment Variable   | Example Value    | Description\n------------------- | ---------------------------- | ---------------- | -----------\n`nerves_fw_active`  | N/A                          | `\"a\"`            | This key holds the prefix that identifies the active firmware metadata. In this example, all keys starting with `\"a.\"` hold information about the running firmware.\n`nerves_fw_devpath` | `NERVES_FW_DEVPATH`          | `\"/dev/mmcblk0\"` | This is the primary storage device for the firmware.\n`nerves_serial_number` | N/A                       | `\"12345abc\"`     | This is a text serial number. See [Serial numbers](#serial_numbers) for details.\n`nerves_fw_validated` | N/A                        | `0`              | Set to \"1\" to indicate that the currently running firmware is valid. (Only supported on some platforms)\n`nerves_fw_autovalidate` | N/A                     | `1`              | Set to \"1\" to indicate that firmware updates are valid without any additional checks.  (Only supported on some platforms)\n`upgrade_available` | N/A                          | `0`              | If using the U-Boot bootloader AND U-Boot's `bootcount` feature, then the `upgrade_available` variable is used instead of `nerves_fw_validated` (it has the opposite meaning)\n`bootcount`         | N/A                          | `1`              | If using the U-Boot bootloader AND U-Boot's `bootcount` feature, then this is the number of times an unvalidated firmware has been booted.\n`bootlimit`         | N/A                          | `1`              | If using the U-Boot bootloader AND U-Boot's `bootcount` feature, then this is the max number of tries for unvalidated firmware.\n\nFirmware-specific Nerves metadata includes the following:\n\nKey                                   | Example Value     | Description\n:------------------------------------ | :---------------- | :----------\n`nerves_fw_application_part0_devpath` | `\"/dev/mmcblk0p3\"` | The block device that contains the application partition\n`nerves_fw_application_part0_fstype`  | `\"ext4\"`          | The application partition's filesystem type\n`nerves_fw_application_part0_target`  | `\"/root\"`         | Where to mount the application partition\n`nerves_fw_architecture`              | `\"arm\"`           | The processor architecture (Not currently used)\n`nerves_fw_author`                    | `\"John Doe\"`      | The person or company that created this firmware\n`nerves_fw_description`               | `\"Stuff\"`         | A description of the project\n`nerves_fw_platform`                  | `\"rpi3\"`          | A name to identify the board that this runs on. It can be checked in the `fwup.conf` before performing an upgrade.\n`nerves_fw_product`                   | `\"My Product\"`    | A product name that may show up in a firmware selection list, for example\n`nerves_fw_version`                   | `\"1.0.0\"`         | The project's version\n`nerves_fw_vcs_identifier`            | `\"bdeead38...\"`   | A `git` SHA or other identifier (optional)\n`nerves_fw_misc`                      | `\"anything...\"`   | Any application info that doesn't fit in another field (optional)\n\nNote that the keys are stored in the environment block prefixed by the firmware\nslot for which they pertain. For example, `a.nerves_fw_description` is the\ndescription for the firmware in the \"A\" slot.\n\nSeveral of the keys can be set in the `mix.exs` file of your main Nerves\nproject. This is the preferred way to set them because it requires the least\namount of effort.\n\nAssuming that your `fwup.conf` respects the `fwup` variable names listed in the\ntable, the keys can also be overridden by setting environment variables at build\ntime. Depending on your project, you may prefer to set them using a customized\n`fwup.conf` configuration file instead.\n\nThe `fwup -m` value shows the key that you'll see if you run `fwup -m -i\nproject.fw` to extract the firmware metadata from the `.fw` file.\n\nKey in `Nerves.Runtime`               | Key in `mix.exs`            | Build Environment Variable            | Key in `fwup -m`\n------------------------------------- | --------------------------- | ------------------------------------- | ----------------\n`nerves_fw_application_part0_devpath` | N/A                         | `NERVES_FW_APPLICATION_PART0_DEVPATH` | N/A\n`nerves_fw_application_part0_fstype`  | N/A                         | `NERVES_FW_APPLICATION_PART0_FSTYPE`  | N/A\n`nerves_fw_application_part0_target`  | N/A                         | `NERVES_FW_APPLICATION_PART0_TARGET`  | N/A\n`nerves_fw_architecture`              | N/A                         | `NERVES_FW_ARCHITECTURE`              | `meta-architecture`\n`nerves_fw_author`                    | `:author`                   | `NERVES_FW_AUTHOR`                    | `meta-author`\n`nerves_fw_description`               | `:description`              | `NERVES_FW_DESCRIPTION`               | `meta-description`\n`nerves_fw_platform`                  | N/A                         | `NERVES_FW_PLATFORM`                  | `meta-platform`\n`nerves_fw_product`                   | `:name`                     | `NERVES_FW_PRODUCT`                   | `meta-product`\n`nerves_fw_version`                   | `:version`                  | `NERVES_FW_VERSION`                   | `meta-version`\n`nerves_fw_vcs_identifier`            | N/A                         | `NERVES_FW_VCS_IDENTIFIER`            | `meta-vcs-identifier`\n`nerves_fw_misc`                      | N/A                         | `NERVES_FW_MISC`                      | `meta-misc`","ref":"readme.html#nerves-system-and-firmware-metadata"},{"type":"extras","title":"Device Reboot and Shutdown - nerves_runtime","doc":"Rebooting, powering-off, and halting a device work by signaling to\n[`erlinit`](https://github.com/nerves-project/erlinit) an intention to shutdown\nand then exiting the Erlang VM by calling `:init.stop/0`. The\n`Nerves.Runtime.reboot/0` and related utilities are helper methods for this.\nOnce they return, the Erlang VM will likely only be available momentarily before\nshutdown. If the OTP applications cannot be stopped within a timeout as\nspecified in the `erlinit.config`, `erlinit` will ungracefully terminate the\nErlang VM.","ref":"readme.html#device-reboot-and-shutdown"},{"type":"extras","title":"Reverting firmware - nerves_runtime","doc":"If you'd like to go back to the previous version of firmware running on a\ndevice, you can do that if the Nerves system supports it. At the IEx prompt,\nrun:\n\n```elixir\niex> Nerves.Runtime.revert\n```\n\nRunning this command manually is useful in development. Production use requires\nmore work to protect against faulty upgrades.\n\nNewer Nerves systems support preventing a revert. This is useful when you've\nloaded a version of firmware that is not meant to be used after it has been\nupgraded. This could be a factory test or an initial firmware that bootstraps\nencrypted firmware storage. See `Nerves.Runtime.FwupOps.prevent_revert/0`.\n\n#","ref":"readme.html#reverting-firmware"},{"type":"extras","title":"Assisted firmware validation and automatic revert - nerves_runtime","doc":"Nerves firmware updates protect against update corruption and power loss\nmidway into the update procedure. However, what happens if the firmware update\ncontains bad code that hangs the device or breaks something important like\nnetworking? Some Nerves systems support tentative runs of new firmware and if\nsomething goes wrong, they'll revert back.\n\nAt a high level, this involves some additional code from the developer that\nknows what constitutes \"working\". This could be \"is it possible to connect to\nthe firmware update server within 5 minutes of boot?\"\n\nHere's the process:\n\n1. New firmware is installed in the normal manner. The `Nerves.Runtime.KV`\n   variable, `nerves_fw_validated` is set to 0. (The systems `fwup.conf` does\n   this)\n2. The system reboots like normal.\n3. The device starts a five minute reboot timer (your code needs to do this if\n   you want to catch hangs or super-slow boots)\n4. The application attempts to make a connection to the firmware update server.\n5. On a good connection, the application sets `nerves_fw_validated` to 1 by\n   calling `Nerves.Runtime.validate_firmware/0` and cancels the reboot timer.\n6. On error, the reboot timer failing, or a hardware watchdog timeout, the\n   system reboots. The bootloader reverts to the previous firmware.\n\nSome Nerves systems support a KV variable called `nerves_fw_autovalidate`. The\nintention of this variable was to make that system support scenarios that\nrequire validate and ones that don't. If the system supports this variable then\nyou should make sure that it is set to 0 (either via a custom fwup.conf or via\nthe provisioning hooks for writing serial numbers to MicroSD cards). Support for\nthe `nerves_fw_autovalidate` variable will likely go away in the future as steps\nare made to make automatic revert on bad firmware a default feature of Nerves\nrather than an add-on.\n\n#","ref":"readme.html#assisted-firmware-validation-and-automatic-revert"},{"type":"extras","title":"U-Boot assisted automatic revert - nerves_runtime","doc":"U-Boot provides a `bootcount` feature that can be used to try out new firmware\nand revert it if it fails. At a high level, it works similar to logic just\ndescribed except that it can attempt a new firmware more than once if desired. This\ncan help if validating a firmware image depends on factors out of your control and\nyou want a few tries to happen before giving up.\n\nTo use this, you need to enable the following U-Boot configuration items:\n\n```text\nCONFIG_BOOTCOUNT_LIMIT=y\nCONFIG_BOOTCOUNT_ENV=y\n```\n\nSee the U-Boot documentation for more information. The gist is to have your\n`bootcmd` handle normal booting and then add an `altbootcmd` to revert the\nfirmware. The firmware update should set the `upgrade_available` U-Boot\nenvironment variable to `\"1\"` to indicate that boot counting should start.\n`Nerves.Runtime.validate_firmware/0` knows about `upgrade_available`, so when\nyou call it to indicate that the firmware is ok, it will set `upgrade_available`\nback to `\"0\"` and reset `\"bootcount\"`.\n\n#","ref":"readme.html#u-boot-assisted-automatic-revert"},{"type":"extras","title":"Best effort automatic revert - nerves_runtime","doc":"Unfortunately, the bootloader for platforms like the Raspberry Pi makes it\ndifficult to implement the above mechanism. The following strategy cannot\nprotect against kernel and early boot issues, but it can still provide value:\n\n1. Upgrade firmware the normal way. Record that the next boot will be the first\n   one in the application data partition.\n2. On the reboot, if this is the first one, record that the boot happened and\n   revert the firmware with `reboot: false`.  If this is not the first boot,\n   carry on.\n3. When you're happy with the new firmware, revert the firmware again with\n   `reboot: false`. I.e., revert the revert. It is critical that `revert` is\n   only called once.\n\nTo make this handle hangs, you'll want to enable a hardware watchdog.","ref":"readme.html#best-effort-automatic-revert"},{"type":"extras","title":"Serial numbers - nerves_runtime","doc":"Finding the serial number of a device is both hardware specific and influenced\nby you and your organization's choices for assigning them (or not). Programs\nshould call `Nerves.Runtime.serial_number/0` to get the serial number.\n\nNerves systems all come with some default way of getting a serial number for a\ndevice. This strategy will likely work for a while, but may not meet your needs\nwhen it comes to production. Nerves uses\n[`boardid`](https://github.com/nerves-project/boardid/) to read serial numbers\nand it can be customized via its `/etc/boardid.config` file. See `boardid` for\nthe mechanisms available. If none of `boardid`'s mechanisms work for you, please\nconsider filing an issue or making a PR, since our history has been that\norganizations tend to use similar mechanisms and it's likely someone else will\nuse it too.\n\nAs a word of caution, many Nerves users write serial numbers in the U-Boot\nenvironment block under the key `nerves_serial_number`. This is supported and\ndocumentation exists for it in many places. While it's very convenient, it has\ndrawbacks - like it's easily modified. It's definitely not the only mechanism.\nThe `boardid.config` file supports trying multiple ways of getting a serial\nnumber to handle hardware changing over the course of development.\n\nSee\n[embedded-elixir](https://embedded-elixir.com/post/2018-06-15-serial_number/)\nfor how to assign serial numbers to devices using the U-Boot environment block\nway.","ref":"readme.html#serial-numbers"},{"type":"extras","title":"Using nerves_runtime in tests - nerves_runtime","doc":"Applications that depend on `nerves_runtime` for accessing provisioning\ninformation from the `Nerves.Runtime.KV` can mock the contents with the included\n`Nerves.Runtime.KVBackend.InMemory` module through the Application config:\n\n```elixir\nconfig :nerves_runtime,\n  kv_backend: {Nerves.Runtime.KVBackend.InMemory, contents: %{\"key\" => \"value\"}}\n```\n\nYou can also create your own module based on the `Nerves.Runtime.KVBackend`\nbehavior and set it to be used in the Application config. In most situations,\nthe provided `Nerves.Runtime.KVBackend.InMemory` should be sufficient, though\nthis would be helpful in cases where you might need to generate the initial\nstate at runtime instead:\n\n```elixir\ndefmodule MyApp.KVBackend.Mock do\n  @behaviour Nerves.Runtime.KVBackend\n\n  @impl Nerves.Runtime.KVBackend\n  def load(_opts) do\n    # initial state\n    %{\n      \"howdy\" => \"partner\",\n      \"dynamic\" => some_runtime_calc_function()\n    }\n  end\n\n  @impl Nerves.Runtime.KVBackend\n  def save(_map, _opts), do: :ok\nend\n\n# Then in config.exs\nconfig :nerves_runtime, :kv_backend, MyApp.KVBackend.Mock\n```","ref":"readme.html#using-nerves_runtime-in-tests"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.13.6 - 2023-11-10 - Changelog","doc":"* Changes\n  * Fix errant return from `Nerves.Runtime.reboot` when using guarded reboots.\n    This resulted in code running after the reboot call that wasn't expected to\n    have been run.\n  * Simplify and harden power off and reboot for edge cases","ref":"changelog.html#v0-13-6-2023-11-10"},{"type":"extras","title":"v0.13.5 - 2023-09-26 - Changelog","doc":"* Changes\n  * Add `Nerves.Runtime.FwupOps` to make it easier to run commands from the\n    `revert.fw` (older) and `ops.fw` (newer) files. The firmware revert logic\n    had always used this, and this makes getting to factory reset and preventing\n    reverts easier. Some operations aren't available on all Nerves systems.\n  * Various documentation and syntactical updates instigated by Elixir 1.15.","ref":"changelog.html#v0-13-5-2023-09-26"},{"type":"extras","title":"v0.13.4 - 2023-04-24 - Changelog","doc":"* Changes\n  * Fixed new compiler warnings with Elixir 1.15\n  * Support the Nerves Heart v2.2 snooze feature.\n    `Nerves.Runtime.Heart.snooze/0` will request that `heart` continues to pet\n    the hardware watchdog and ignore lack of updates from Erlang for a short\n    amount of time to allow debug of conditions that would otherwise be\n    interrupted by a watchdog timeout.\n  * Report errors from Heart functions rather than hang forever when Erlang\n    `:heart` is unresponsive.\n  * Support additional status from Nerves Heart v2.2 to report the initial\n    bootup grace period timeout and snooze timeout remaining.","ref":"changelog.html#v0-13-4-2023-04-24"},{"type":"extras","title":"v0.13.3 - 2022-11-25 - Changelog","doc":"* Changes\n  * Fixed poweroff with Nerves Heart v2.0 (be sure to use Nerves Heart v2.0.2 as\n    well). Thanks to @amclain for finding and fixing this.","ref":"changelog.html#v0-13-3-2022-11-25"},{"type":"extras","title":"v0.13.2 - 2022-11-22 - Changelog","doc":"Support Nerves Heart v2.0.0 attributes and features. Nerves Heart v2.0.0 is a\nmajor update that addresses rare cases where devices would fail to reboot or\ndetect an issue. It also adds helpful statistics. See the docs for details.\nNerves Heart v1.x versions are still supported.\n\n* Changes\n  * Support guarded reboot and poweroff if Nerves Heart 2.0 is available.\n    Reboots and poweroffs still gracefully stop the VM, but they now don't\n    require Busybox `reboot` or `poweroff`, and they stop petting the hardware\n    watchdog to set a hard limit.\n  * Support Nerves Heart initialization completion notifications. This lets you\n    protect the time between boot and `:heart.set_callback/1` should something\n    happen that prevents the callback from being set. This addresses an issue\n    where the VM would think everything is great since it wasn't calling the\n    callback to find out that it was not.","ref":"changelog.html#v0-13-2-2022-11-22"},{"type":"extras","title":"v0.13.1 - 2022-07-06 - Changelog","doc":"* Changes\n  * Handle another way key-value stores were being mocked pre-v0.13.0.\n  * Fix default `boardid_path` in mix.exs","ref":"changelog.html#v0-13-1-2022-07-06"},{"type":"extras","title":"v0.13.0 - 2022-07-05 - Changelog","doc":"This update replaces `Nerves.Runtime.target/0` with\n`Nerves.Runtime.mix_target/0`. It is not believed that the former function was\nused much, but if you do use it, note the return value change in `mix_target/0`.\n\n* Changes\n  * Support customizable key-value store backends so that it's possible to store\n    firmware and board metadata outside of U-Boot environment blocks. See the\n    `:kv_backend` configuration option.\n  * Support changing the firmware revert script path (see `:revert_fw_path`)\n  * Decide host vs. target differences at compile time to avoid target-only\n    or host-only logic and dependencies existing unnecessarily at run-time.","ref":"changelog.html#v0-13-0-2022-07-05"},{"type":"extras","title":"v0.12.0 - 2022-06-03 - Changelog","doc":"This is a major update to `nerves_runtime` that removes SystemRegistry.\nSystemRegistry has been disabled by default for years, but it could be\nre-enabled if device insertion and removal events were needed. That\nfunctionality has been moved to\n[`nerves_uevent`](https://hexdocs.pm/nerves_uevent/readme.html) and updated to\nuse the `property_table` library used by VintageNet.\n\nElixir 1.11 is the minimum supported Elixir version now.\n\nTo upgrade from prior versions of `nerves_runtime`:\n\n1. If you're using `:system_registry` at all, please review the `:nerves_uevent`\n   documentation for new library.\n2. Remove `config :nerves_runtime, :kernel, use_system_registry: false` from\n   your `target.exs`. It's not used any more.\n3. If you had `config :nerves_runtime, :kernel, autoload_modules: false` in your\n   `target.exs`, change it to `config :nerves_uevent, autoload_modules: false`.\n\n* Bug fixes\n  * Run `reboot` and `poweroff` sequences in their own Erlang process. This\n    fixes a bug were the process that calls `reboot` gets killed partway through\n    the reboot process and the system doesn't actually reboot.\n\n* Changes\n  * Kernel logging and syslog monitoring has been moved to\n    [`nerves_logging`](https://hexdocs.pm/nerves_logging/readme.html). The\n    functionality is the same as before, but it's now possible to use without\n    `nerves_runtime`.\n  * Added convenience routines for getting status from\n    [`nerves_heart`](https://github.com/nerves-project/nerves_heart).","ref":"changelog.html#v0-12-0-2022-06-03"},{"type":"extras","title":"v0.11.10 - 2022-3-17 - Changelog","doc":"* Bug fixes\n  * Update the initial device scan to be deterministic and to trigger events for\n    containers before the devices they contain. This fixes a subtle issue that\n    prevented RT5379 WiFi module drivers from automatically being loaded.\n  * Batch up modprobe invocations","ref":"changelog.html#v0-11-10-2022-3-17"},{"type":"extras","title":"v0.11.9 - 2022-3-6 - Changelog","doc":"* Changes\n  * Save shell history on first boot. Previously, Erlang would open up the shell\n    history file before the data partition was formatted and wouldn't retry.\n    Nerves.Runtime now resets shell history so that it retries.\n  * Call modprobe to load kernel modules to C to reduce boot time.","ref":"changelog.html#v0-11-9-2022-3-6"},{"type":"extras","title":"v0.11.8 - 2021-10-25 - Changelog","doc":"* Changes\n  * Update the mix dependencies to allow `uboot_env v1.0.0` to be used.","ref":"changelog.html#v0-11-8-2021-10-25"},{"type":"extras","title":"v0.11.7 - Changelog","doc":"* Bug fixes\n  * `Nerves.Runtime.firmware_valid?/0` would return that the firmware wasn't\n    validated when the validation feature wasn't in use. This was confusing\n    since firmware is assumed valid when the feature is off.","ref":"changelog.html#v0-11-7"},{"type":"extras","title":"v0.11.6 - Changelog","doc":"* New features\n  * Added support for implementing auto-revert logic using U-Boot's\n    bootcount/upgrade_available feature. This can simplify U-Boot scripts. See\n    the README.md for details.\n\n* Bug fixes\n  * Specify \"-f\" to force f2fs filesystem formats like those of other\n    filesystems. Thank to Eric Rauer for catching this oversight.","ref":"changelog.html#v0-11-6"},{"type":"extras","title":"v0.11.5 - Changelog","doc":"* Updates\n  * If `/etc/sysctl.conf` is present, run `sysctl` to load and set the kernel\n    configuration parameters in it.\n  * Improve C compilation messages and error help","ref":"changelog.html#v0-11-5"},{"type":"extras","title":"v0.11.4 - Changelog","doc":"* Updates\n  * Added `Nerves.Runtime.firmware_valid?/0` to easily check whether the\n    firmware has been marked valid for systems that auto-rollback.","ref":"changelog.html#v0-11-4"},{"type":"extras","title":"v0.11.3 - Changelog","doc":"* Updates\n  * Support `uboot_env` v0.3.0. This version of `uboot_env` has backwards\n    incompatible changes, but they don't affect `nerves_runtime`, so the\n    `mix.exs` deps spec allows it now.\n  * Tightened up the specs on `Nerves.Runtime.KV` functions.\n\nSupport for OTP 20 was removed. `uboot_env` v0.3.0 requires OTP 21 and later. If\nyou still are using OTP 20, lock the version of `uboot_env` to `~> 0.2.0` in\nyour `mix.exs`.","ref":"changelog.html#v0-11-3"},{"type":"extras","title":"v0.11.2 - Changelog","doc":"* New features\n  * Added `Nerves.Runtime.validate_firmware/0` for validating firmware on\n    systems that auto-rollback. This *only* abstracts the setting of the\n    `nerves_fw_validated` key. It doesn't add any new functionality. However, it\n    will enable auto-rollback to be added to Nerves systems in a consistent\n    manner in the future and allow for platform-specific variations without\n    impacting application code.","ref":"changelog.html#v0-11-2"},{"type":"extras","title":"v0.11.1 - Changelog","doc":"* Bug fixes\n  * Reap zombie process that was created by the uevent port helper.\n  * Support `uboot_env` v0.2.0 to reduce memory garbage that's created when\n    reading and writing U-Boot environment blocks","ref":"changelog.html#v0-11-1"},{"type":"extras","title":"v0.11.0 - Changelog","doc":"* New features\n  * Add `Nerves.Runtime.serial_number/0`. It will call out to the underlying\n    system to return the device's serial number however it's stored.\n  * Add a fallback to `haveged` for systems that don't have hardware random\n    number generators or otherwise can't use `rngd`.","ref":"changelog.html#v0-11-0"},{"type":"extras","title":"v0.10.3 - Changelog","doc":"* Bug fixes\n  * Fix potential process accumulation from the kmsg_tailer process ignoring\n    stdin being closed on it.\n  * Tightened deps to avoid combinations that would be difficult to support","ref":"changelog.html#v0-10-3"},{"type":"extras","title":"v0.10.2 - Changelog","doc":"* Bug fixes\n  * Fix off-by-one error when processing uevent messages with device paths\n    longer than 16 segments. This also bumps the max number of segments to 32.\n  * Fix logger message about `rngd` failing when it was successful\n  * Log errors when required commands aren't available rather than raising. It\n    turned out that raising was disabling logging and that was making it hard to\n    figure out the root cause.\n\n* Enhancements\n  * Switch from parsing `/proc/kmsg` to `/dev/kmsg` for kernel messages. The\n    device interface supplies a little more information and is unaffected by\n    other programs reading from it. This change refactored syslog/kmsg parsing\n    to improve test coverage. This is considered to be an internal API. If you\n    were using it, you will need to update your code.","ref":"changelog.html#v0-10-2"},{"type":"extras","title":"v0.10.1 - Changelog","doc":"* Bug fixes\n  * Fix exception on init when mounting the application data partition.\n    This addresses an issue where `:nerves_runtime` would exit due to an\n    unmatched call to `System.cmd/3`.","ref":"changelog.html#v0-10-1"},{"type":"extras","title":"v0.10.0 - Changelog","doc":"* New feature\n  * Added `Nerves.Runtime.KV.put` and `Nerves.Runtime.KV.put_active` to support\n    setting environment. This removes the need to run `fw_setenv` and also\n    updates the cached key/value pairs. Thanks to Troels Brødsgaard for\n    implementing.","ref":"changelog.html#v0-10-0"},{"type":"extras","title":"v0.9.5 - Changelog","doc":"* Bug fixes\n  * Fix C compiler error (PATH_MAX undeclared) on x86_64/muslc systems","ref":"changelog.html#v0-9-5"},{"type":"extras","title":"v0.9.4 - Changelog","doc":"* Enhancements\n  * Reduced number of syscalls needed for transferring device detection events\n    (uevents) to Elixir by batching them\n  * Improved start up performance by moving initial device enumeration to C\n  * Handle uevent overloads by dropping messages rather than crashing. Uevent\n    bursts are handled better by the new batching, but can be more severe due to\n    faster enumeration. Both dropping and crashing have drawbacks, but dropping\n    made it possible to recover on a 32-processor machine with many peripherals.","ref":"changelog.html#v0-9-4"},{"type":"extras","title":"v0.9.3 - Changelog","doc":"* Enhancements\n  * Move C build products to under `_build`","ref":"changelog.html#v0-9-3"},{"type":"extras","title":"v0.9.2 - Changelog","doc":"* Enhancements\n  * Support disabling SystemRegistry integration. This is not a recommended\n    setting and should only be used on slow devices that don't need device\n    insertion/removal notifications.","ref":"changelog.html#v0-9-2"},{"type":"extras","title":"v0.9.1 - Changelog","doc":"* Enhancements\n  * Filter out `synth_uuid` from uevent reports since it's not supported by the\n    current uevent handling code.\n  * Further reduce garbage produced by processing uevent reports","ref":"changelog.html#v0-9-1"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"The Nerves Runtime Helpers have been extracted and are now part of\n[Toolshed](https://hex.pm/packages/toolshed). The helpers included things like\n`cmd/1` and `reboot/0` that you could run at the IEx prompt. Toolshed is not\nincluded as a dependency. If you would like it, please add `toolshed` to your\napplication dependencies. You'll find that Toolshed contains more helpers. It is\nalso easier for us to maintain since changes to the helpers no longer affect\nall Nerves projects.\n\n* Enhancements\n  * Further optimize enumeration of devices at boot. This fixes an issue where\n    uniprocessor boards (like the RPi Zero and BBB) would appear to stall\n    momentarily on boot.\n  * The U-Boot environment processing code has been factored out of\n    `nerves_runtime` so that it can be used independently from Nerves. It can be\n    found at [uboot_env](https://hex.pm/packages/uboot_env).","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"* Enhancements\n  * Optimized enumeration of devices at boot. You will likely notice devices\n    becoming available more quickly. This may uncover race conditions in\n    application initialization code.\n  * The `syslog` monitoring code has been rewritten in pure Elixir. This\n    component captures log messages sent by C programs using the `syslog` system\n    call so that they can be handled by Elixir Logger backends.\n  * The kmsg log monitor still requires C, but the C code is simpler now that it\n    doesn't process `syslog` messages as well.\n  * The Linux `uevent` processing code was simplified and the C to Elixir\n    communications refactored to minimize processing of events.\n\n* Bug fixes\n  * Fixed a race condition with the Linux kernel and processing `/sys/devices`\n    that could cause an exception during device enumeration.\n  * Syslog messages w/o terminating newlines are logged now.","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"* Enhancements\n  * Documentation updates to `nerves_serial_number`, `nerves_validated`, and\n    `nerves_autovalidate`.\n  * Added helper function `Nerves.Runtime.KV.UBootEnv.put/2` for writing to the\n    UBoot env. This is useful for setting provisioning information at runtime.\n  * Added the ability to mock the contents of `Nerves.Runtime.KV` for use in\n    test and dev. The contents can be set in your application config.\n\n      config :nerves_runtime, :modules, [\n        {Nerves.Runtime.KV.Mock, %{\"key\" => \"value\"}}\n      ]\n\n* Bug fixes\n  * Kernel uevent `change` messages no longer cause modifications to\n    system_registry.","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"v0.6.5 - Changelog","doc":"Update dependencies to only include `dialyxir` for `[:dev, :test]`, preventing\nit from being distributed in the with the hex package. This addresses an issue\nwhere `dialyxir` and its dependencies would be included in the applications list\nwhen producing the OTP release and cause `:wx` to raise because the target\nversion of `erts` was compiled without it.","ref":"changelog.html#v0-6-5"},{"type":"extras","title":"v0.6.4 - Changelog","doc":"* Bug fixes\n  * Fix U-Boot environment load issue and add unit tests to cover environment\n    block generation using either U-Boot tools or fwup\n  * Load rngd if available. If it's available, this greatly shortens the time\n    for the Linux kernel's random number entropy pool to initialize. This\n    improves boot time for applications that need random numbers right away.","ref":"changelog.html#v0-6-4"},{"type":"extras","title":"v0.6.3 - Changelog","doc":"* Bug fixes\n  * Fix issue with parsing fw_env.config files with space separated values.","ref":"changelog.html#v0-6-3"},{"type":"extras","title":"v0.6.2 - Changelog","doc":"* Enhancements\n  * Updates to docs and typespecs.\n\n* Bug fixes\n  * Read the U-Boot environment directly using :file if possible. (OTP-21)\n    This fixes an issue with `fw_printenv` where multi-line values cause the\n    output to be unparseable.","ref":"changelog.html#v0-6-2"},{"type":"extras","title":"v0.6.1 - Changelog","doc":"* Bug fixes\n  * Log the output of system commands to that they're easier to review\n  * Support mounting `f2fs` (requires support in the Nerves system to work)","ref":"changelog.html#v0-6-1"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"* New features\n  * Forward operating system messages from `/dev/log` and `/proc/kmsg` to\n    Elixir's `Logger`. If the log volume is too much to the console, consider\n    replacing the console logger with `ring_logger` or another logger backend.\n\n* Bug Fixes\n  * `cmd/1` helper improvements to interactively print output from long running\n    commands.\n  * `Nerves.Runtime.revert` would always reboot even if told not to.\n  * Cleanup throughout to improve docs, formatting, and newer Elixir stylistic\n    conventions\n  * Remove scary suid printout when crosscompiling since nothing setuid related\n    was going on.","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"v0.5.2 - Changelog","doc":"* Enhancements\n  * Added `Nerves.Runtime.revert` to revert the device to the inactive\n    firmware. To use this, the Nerves system needs to include revert\n    instructions. This is currently being implemented.\n\n* Bug Fixes\n  * Setuid the `uevent` port binary for debugging on the host.","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"v0.5.1 - Changelog","doc":"* Bug Fixes\n  * Split device attributes into only two parts","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"* The Nerves runtime shell (the bash shell-like shell from CTRL+G) has been\n  moved to a separate project so that it can evolve independently from\n  `nerves_runtime`. As such, it's no longer available, but see\n  the `nerves_runtime_shell` project to include it again.\n\n* Enhancements\n  * Force application partition UUID. The UUID has previously been random. The\n    main reason for change is to avoid a delay when waiting for the urandom\n    pool to initialize. Having a known UUID for the application partition may\n    come in handy in the future, though.","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"v0.4.4 - Changelog","doc":"* Enhancements\n  * Added installation helper for `Nerves.Runtime.Helpers`\n  * Privatized methods in KV so that they don't show up in the tab complete\n    and help screens.","ref":"changelog.html#v0-4-4"},{"type":"extras","title":"v0.4.3 - Changelog","doc":"* Enhancements\n  * The `Nerves.Runtime.Helpers` module provides a number of functions that are\n    useful when working at the IEx prompt on a target.","ref":"changelog.html#v0-4-3"},{"type":"extras","title":"v0.4.2 - Changelog","doc":"* Enhancements\n  * Makefile only builds for Linux hosts or cross compile environments. Allows\n    package to compile on other platforms.","ref":"changelog.html#v0-4-2"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"* Bug Fixes\n  * Fixed issue with the order of args being passed to mkfs","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"* Enhancements\n  * Loosen dependency requirements on SystemRegistry","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"* Bug Fixes\n  * Increased erl_cmd buffer size to 2048 to prevent segfaults with uevents for\n    devices with many attributes.","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"* Enhancements\n  * Removed GenStage in favor of SystemRegistry\n  * Added KV firmware variable key value store\n  * Added Init worker for initializing the application partition","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"* Enhancements\n  * Moved hardware abstraction layer to separate project for further\n    development\n  * Start the shell using the name `sh` instead of `'Elixir.Nerves.Runtime.Shell'`","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.2 - Changelog","doc":"* Bug fixes\n  * Cleaned up IO\n  * Rename host to sh","ref":"changelog.html#v0-1-2"}],"content_type":"text/markdown"}